/**
 * License Agreement.
 *
 *  JBoss RichFaces - Ajax4jsf Component Library
 *
 * Copyright (C) 2007  Exadel, Inc.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License version 2.1 as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */
package org.richfaces.realworld.service;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import javax.ejb.Stateless;
import javax.persistence.EntityManager;

import org.jboss.seam.annotations.AutoCreate;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Out;
import org.richfaces.realworld.domain.Image;
import org.richfaces.realworld.domain.Shelf;
import org.richfaces.realworld.domain.User;

@Name("shelfAction")
@Stateless
@AutoCreate
public class ShelfAction implements IShelfAction {

	@In(value="entityManager")
	EntityManager em;
	@In FlushStrategy flushStrategy;
	@In @Out
	private User user;

	public void addShelf(Shelf shelf) {
		em.persist(shelf);
		user.addShelf(shelf);
		if(flushStrategy.isDatabaseStoreStrategy()){
			em.flush();
		}
	}
	
	public void deleteShelf(Shelf shelf){
		user.removeShelf(shelf);
		em.remove(shelf);
		if(flushStrategy.isDatabaseStoreStrategy()){
			em.flush();
		}
	}
	
	public void editShelf(Shelf shelf){
		if(flushStrategy.isDatabaseStoreStrategy()){
			em.flush();
		}
	}
	
	public List<Shelf> getShelfs(User user) {
		//TODO nick - preserve shelves ordering? Maybe use TreeSet with Comparator?
		Set<Shelf> sh = new HashSet<Shelf>();
		List<Shelf> shelfs = em.createQuery("from Shelf s where s.shared=true")
		.getResultList();
		sh.addAll(shelfs);
		sh.addAll(user.getShelfs());
		shelfs.clear();
		shelfs.addAll(sh);
		return shelfs;
	}

	public List<Image> getUnvisitedImages(Shelf shelf){
		//TODO nick - use data from user
		return (List<Image>)em.createQuery("from Image i where i.album.shelf=:shelf and i.uploaded > :date").setParameter("shelf", shelf).setParameter("date", getDate()).getResultList();
	}
	
	private Date getDate() {
		//TODO nick - copy-pasted from AlbumAction, see TODOs there
		Calendar c = Calendar.getInstance();
		c.add(Calendar.DAY_OF_YEAR, -15); 
		return c.getTime();
	}
}
